/* $Id: LeastSquares.java,v 1.1 2005/07/28 07:48:31 mit Exp $ */
/* LeastSquares.java v */
import java.text.DecimalFormat;

public class LeastSquares{
    private String keyname;
    private int key1; // 0~87
    private int key2;
    private double length1;
    private double length2;
    private double gauge1;
    private double gauge2;
    private double lsmA;
    private double lsmB;

    private Bante b = new Bante();
    private DecimalFormat df1 = new DecimalFormat("#0.0##");

    private final int KB = 88;
    private final int Oct = 12;
    private final double AA = 440.0; 

    public LeastSquares() {
        setWire("", 0, 0, 0, 0, 0, 0);
    }
    public LeastSquares(String nm, int key1, double l1, double g1, 
			int key2, double l2, double g2) {
        setWire(nm, key1, l1, g1, key2, l2, g2);
    }
    public LeastSquares(String nm, double[] data) {
	setWire(nm, data[0], data[1], data[2], data[3],data[4],data[5]);
    }
    public void setWire(String nm, double k1, double l1, double g1, 
			double k2, double l2, double g2) {
	keyname = nm;
	key1 = (int)k1;
	length1 = l1;
	gauge1 = g1;
	key2 = (int)k2;
	length2 = l2;
	gauge2 = g2;
    }
    public void setWire(String nm, int k1, double l1, double g1, 
			int k2, double l2, double g2) {
	keyname = nm;
	key1 = k1;
	length1 = l1;
	gauge1 = g1;
	key2 = k2;
	length2 = l2;
	gauge2 = g2;
    }

    public String getName() {
	return keyname;
    }
    public double getInhar1() {
	return this.getInhar(key1, length1, gauge1);
    }
    public double getInhar2() {
	return this.getInhar(key2, length2, gauge2);
    }
    public double getInhar(int key, double len, double ga) {
	return 3.3*Math.pow(10,13)*Math.pow(this.getDiameter(ga)/10,2)/
	    Math.pow(len/10,4)/Math.pow(this.getFreq(key),2);
    }
    public double getDiameter(double ga) {
        return (ga < 12 && ga > 0.1) ? ga : b.getDiameter(ga);
    }
    public double getLSMA() {
	if (lsmA == 0)
	    calcLSM();

	return lsmA;
    }
    public double getLSMB() {
	if (lsmB == 0)
	    calcLSM();

	return lsmB;
    }
    public void calcLSM() {
	double s1 = 0, s2 = 0, xb = 0, yb = 0;
	double y;
	double[] tmp = new double[KB];
  
	tmp[key1] = this.getInhar1();
	tmp[key2] = this.getInhar2();
	int x, n = 0;
	for (int i = 0; i < KB; i++) {
	    if (tmp[i] <= 0)
		continue;
	    x = i+1; // 0~87
	    y = Math.log(tmp[i]);
	    s1 += x*y;
	    s2 += x*x;
	    xb += x;
	    yb += y;
	    n++;
	}
	s1 /= n;
	s2 /= n;
	xb /= n;
	yb /= n;
	lsmA = Math.exp((s2*yb-s1*xb)/(s2-xb*xb));
	lsmB = (s1-yb*xb)/(s2-xb*xb);
    }

    public String toString() {
	StringBuffer sb = new StringBuffer(48);
	sb.append(keyname);
	sb.append(" "+dform(this.getInhar1()));
	sb.append(" "+dform(this.getInhar2()));
	sb.append(" "+dform(this.getLSMB()));
	//sb.append("\n");

        return sb.toString();
    }
    public String dform(double n) {
        return df1.format((float)n);
    }
    public double getFreq(int key) { // 0~87
	key++;
	return AA*Math.pow(2, (key-49)/(double)Oct);
    }
}