/* $Id: ScaleCalculation.java,v 1.7 2013/12/04 04:58:43 mit Exp $ */
/* ScaleCalculation.java v0.2 (design) */

public class ScaleCalculation
{
    private int KB;
    private int BSTT; // ݥ
    private int BEND; // "
    private int A49;  // "
    private int C88;  // "
    private int BHW;
    private double[] SPoint;
    private double[] Ibase;
    private boolean Isw = true;
    private Bante B;
    private Wire[] TG;
    private TuningKeyboard TK;
    private TuningFormula2 TF2;

    public ScaleCalculation() { }
    public ScaleCalculation(int a, int b, int c, int d,
			    TuningKeyboard tk, Wire[] tg, 
			    TuningFormula2 tf, boolean isw) {
        set_sc(a, b, c, d, tk, tg, tf, isw);
    }
    public ScaleCalculation(int[] bs, TuningKeyboard tk, Wire[] tg, 
			    TuningFormula2 tf, boolean isw) {
        set_sc(bs[0], bs[1], bs[2], bs[3], tk, tg, tf, isw);
    }
    public void set_sc(int a, int b, int c, int d,
		       TuningKeyboard tk, Wire[] tg, 
		       TuningFormula2 tf, boolean isw) {
        BSTT = a;
        BEND = b;
        A49 = c;
        C88 = d;

        BHW = BEND+1;
        KB = C88+1;
        SPoint = new double[KB];
        Ibase = new double[KB];
	B = new Bante();
	TG = tg;
        TK = tk;
	TF2 = tf;
	Isw = isw;

        set_spoint();
    }
    public double getIbase(int key) {
        return Ibase[key];
    }
    public double getSPoint(int key) {
        return SPoint[key];
    }
    public LeastSquares getLSM(Wire[] g) {
	LeastSquares lsm =
	    new LeastSquares("Design",
			     A49, g[A49].getLength(), g[A49].getGauge(),
			     C88, g[C88].getLength(), g[C88].getGauge());
	return lsm;
    }
    public void setIsw(boolean sw) {
	Isw = sw;
    }
    //
    public void calcWire() {
	set_chord();
	set_spoint();
	set_ibase();

	calc_bante();
        calc_copper();
	calc_chord();
	calc_base_wire();
    }
    public void set_spoint() { // Ǹ
        int c88 = 22, a49 = 8, sa = C88-A49;
	double val = (double)c88/a49;

	for (int i = C88; i > A49; i--)
	    SPoint[i] = get_hi(i, a49, val, sa);

	for (int i = A49; i >= 0; i--)   // 1/8
	    SPoint[i] = a49;
    }
    //
    public void set_chord() { // c88~a49
        double c88 = TG[C88].getLength();
	double a49 = TG[A49].getLength();
        if (c88 < 1 || a49 < 1)          // 0Ϸ׻
            return;

	int sa = C88-A49;
	double la, val = c88/a49;
	for (int i = C88; i > BEND; i--) {
	    la = get_hi(i, a49, val, sa);
	    TK.setLength(i, la);
	}
    }
    public void set_ibase() {
        if (Isw) { // straight
	    Ibase[A49] = TG[A49].getInhar();
	    Ibase[C88] = TG[C88].getInhar();

	    int sa = C88-A49;
	    double val = Ibase[C88]/Ibase[A49];
	    for (int i = 0; i < KB; i++) {
		Ibase[i] = get_hi(i, Ibase[A49], val, sa);
	    }

	    Ibase[BSTT] = TG[BSTT].getInhar();
	    Ibase[BEND] = TG[BEND].getInhar();
	    sa = BSTT-BEND;
	    val = Ibase[BSTT]/Ibase[BEND];
	    for (int i = BSTT; i <= BEND; i++) {
		Ibase[i] = get_him(i, Ibase[BSTT], val, sa);
	    }
        } else { // curve
	    LeastSquares ls = getLSM(TG); // 49-88
	    TF2.setGrade(ls.getLSMB());
	    TF2.setPitch(TG[A49].getInhar());
	    TF2.setWound(BEND);
	    Ibase = TF2.getInha();
	}
    }
    public double get_hi(int key, double pit, double val, double sa) {
	return pit*Math.pow(val, (key-48)/sa); // 49-1
    }
    public double get_him(int key, double pit, double val, double sa) {
	return pit*Math.pow(val, key/sa);
    }
    public void calc_bante() { // ¤ּζ
	double c88 = TG[C88].getDia();
	double a49 = TG[A49].getDia();
	int sa = C88-A49;
	double d, val = c88/a49;
	for (int i = C88; i > BEND; i--) {
	    d = get_hi(i, a49, val, sa);
	    TK.setGauge(i, B.getNearb(d));
	}

	double bed = TG[BEND].getDia();
	double bst = TG[BSTT].getDia();
	sa = BSTT-BEND;
	val = bst/bed;
	for (int i = BSTT; i <= BEND; i++) {
	    d = get_him(i, bst, val, sa);
	    TK.setGauge(i, B.getNearb(d));
	}
    }
    public void calc_base_wire() { // inha.鸹Ĺ
	double bed = TG[BEND].getLength();
	double bst = TG[BSTT].getLength();
	int sa = BSTT-BEND;
	double l, val = bst/bed;

	for (int i = BSTT; i <= BEND; i++) {
	    l = get_him(i, bed, val, sa);
	    TK.setLength(i, l);
	}

	double ih, lng, d = 1.0; // diff
	for (int i = BSTT; i <= BEND; i++) {
	    ih = TK.getInha(i);

	    while (ih < Ibase[i]) {
		lng = TK.getLength(i);
		TK.setLength(i, lng-d);
		ih = TK.getInha(i);
	    }
	    while (ih > Ibase[i]) {
		lng = TK.getLength(i);
		TK.setLength(i, lng+d);
		ih = TK.getInha(i);
	    }
	}
    }
    public void calc_chord() { // inha.鸹Ĺ׻
	for (int i = C88; i > BEND; i--)
	    TK.setLength(i, get_length(i));
    }
    public double get_length(int key) { // Inha鸹Ĺεշ׻
	return Math.pow(3.3*Math.pow(10,15)*Math.pow(TK.getDia(key),2)/
			(Ibase[key]*Math.pow(TK.getFreq(key),2)), (1.0/4));
    }
    public double get_dia(int n) { //  Inhaȥ
	return Math.sqrt(Ibase[n]*Math.pow(TK.getLength(n),4)*
			 Math.pow(TK.getFreq(n),2)/(3.3*Math.pow(10,15)));
    }
    public void calc_copper() { // copper 0.2 ~ 1.9 step 0.05 //
     	double le = TG[BEND].getLength();
	double ls = TG[BSTT].getLength();
	if (le < 1 || ls < 1)  // 0Ϸ׻
	    return;

	int sa = BSTT-BEND;
	double ll, val = ls/le;
	if (TG[BSTT].getCopper() > 0 || TG[BEND].getCopper() > 0) {
	    double cs = TG[BSTT].getalld(); // 100%
	    double ce = TG[BEND].getalld();
            double d, c, cc, vac = cs/ce;
            for (int i = BSTT; i <= BEND; i++) {
		ll = get_him(i, le, val, sa); // le
		cc = get_him(i, cs, vac, sa);
		d = TK.getDiameter(i);
                c = get_Copper((cc-d)/2); // Ƽ
		TK.setWire(i, ll, B.getBante(d), c);
            }
        } else {        // ξü0ʥη׻ԥ
	    for (int i = BSTT; i <= BEND; i++) {
		ll = get_him(i, le, val, sa); // le
		TK.setWire(i, ll, TK.getGauge(i), 0);
	    }
        }
    }
    public double get_Copper(double d) {
        double copp = 0.2;
        if (d > 1.9)
            copp = 1.9; 
        copp = get_cop(copp, d);
        return copp;
    }
    public double get_cop(double stt, double d) {
        double cop = stt, copp = 0;
        double dif1, dif0 = d-copp;
        for (int i = 1; i < 35; i++) {
            dif1 = Math.abs(d-cop);
            if (dif1 < dif0) {
                dif0 = dif1;
                copp = cop;
            }
            cop += 0.05;
        }
        return copp;
    }
}