/* $Id: TuningFormula2.java,v 1.4 2014/02/26 05:01:16 mit Exp $
/* TuningFormula2 v0.4 for Design*/
import java.text.DecimalFormat;

public class TuningFormula2 {
    private double GRADE = 0.087;  //
    private double A49I = 0.55;   // inharmo.
    private int WOUND = 27;      // wound wire
    private double NN;     // for sinh
    private double AA;     // for e
    private double BB;     // "
    private double BEAT;
    private double[] Cent;
    private double[] Inha; // Cosh
    private double[] Sinh;
    private double[] Ee;
    private double[] IH; // Inha.

    private final int ITV = 24; // 2octave tune
    private final Interval I = new Interval();
    private final int ITN = I.getIinte(ITV);
    private final int KB = 88;
    private final int KC = KB/2-1; // octave˹碌
    private final int PITCH = 440;
    private final int OCTAVE = 12;
    private final int CENT = 1200;
    private final double RevO = 1.0/OCTAVE;
    private DecimalFormat df1 = new DecimalFormat("#0.0##");

    public double[] getInharmonicity(double[] inha) {
	IH = inha;
	tuneCosh();

	return getInha();
    } /* .(Wireľ)饤϶ */
    public double[] getCurve(double be) {
	BEAT = be;
	tuneSinh();
	tuneE();

	return Cent;
    } /* ĴΧ */
    //-----

    public double[] getInha() {
	Inha = makeCosh(WOUND, A49I, GRADE);
	return Inha;
    } /* ͤ饤϶ */
    public double[] getCent() {
	return Cent;
    }
    public double getGrade() {
        return GRADE;
    }
    public double getPitch() {
        return A49I;
    }
    public int getWw() {
        return WOUND;
    }
    public void setCent(int key, double ce) {
        Cent[key] = ce;
    }
    public void addCent(int key, double ce) {
        Cent[key] += ce;
    }
    //---
    public double[] getSinh() {
	return Sinh;
    }
    public double getNn() {
	return NN;
    }
    //---
    public double[] getE() {
	return Ee;
    }
    public double getAa() {
	return AA;
    }
    public double getBb() {
	return BB;
    }
    public int getItv() {
	return ITV;
    }
    public int getItn() {
	return ITN;
    }
    //---
    public void setGrade(double g) {
	GRADE = g;
    }
    public void setPitch(double p) {
	A49I = p;
    }
    public void setWound(int w) {
	WOUND = w;
    }

    //--- cosh
    public void tuneCosh() {
	double i0 = 0, g0 = 0;
	int w0 = 0;

	while(true) {
	    calc_inha();
	    calc_grade();
	    calc_ww();

	    if (i0 == A49I & g0 == GRADE & w0 == WOUND)
                break;

            i0 = A49I;
            g0 = GRADE;
            w0 = WOUND;
        }
    }
    public void calc_grade() {
        double di, df;
        double sa = 0.001;
        double d0 = calc_grade(GRADE);
        double dp = calc_grade(GRADE+sa);
        double dm = calc_grade(GRADE-sa);
        if (dp < d0)
            sa = sa;
        else if (dm <= d0)
            sa = -sa;
        df = GRADE;
        while(true) {
            df += sa;
            di = calc_grade(df);

            if (di < d0) {
                GRADE = df;
                d0 = di;
            } else
                break;
	}
    }
    public double calc_grade(double gr) {
        Inha = makeCosh(WOUND, A49I, gr);

        return diff_h();
    }
    public double diff_h() {
        double df = 0;
        for (int i = 0; i < KB; i++)
            df += Math.abs(calc_log(IH[i])-calc_log(Inha[i]));

        return df;
    }
    public double calc_log(double ih) {
        return log_base(ih);
    }
    public double log_base(double value) {
        return Math.log(value)/Math.log(10);
    }
    public void calc_inha() {
        double di, df;
        double d0 = calc_inha(A49I);
        double ih = Math.abs(log_base(A49I)-log_base(IH[48]));
        double sa = (A49I > 2.0) ? 0.5 : (ih > 1.4) ? 0.1 : 0.01;
        double dp = calc_inha(A49I+sa);
        double dm = calc_inha(A49I-sa);
        if (d0 > dp)
            sa = sa;
        else if (d0 >= dm)
            sa = -sa;
        df = A49I;
        while(true) {
            df += sa;
            di = calc_inha(df);
            if (di < d0) {
                A49I = df;
                d0 = di;
            } else
                break;
        }
    }
    public double calc_inha(double ih) {
        Inha = makeCosh(WOUND, ih, GRADE);

        return diff_h();
    }
    public void calc_ww() {
        double di;
        int df, sa = 1;
        if (WOUND < 0)
            WOUND = 1;
        else if (WOUND > 44)
            WOUND = 44;
        double d0 = calc_wound(WOUND);
        double dp = calc_wound(WOUND+sa);
        double dm = calc_wound(WOUND-sa);
        if (dp < d0)
            sa = sa;
        else if (dm <= d0)
            sa = -sa;
        df = WOUND;
        while(true) {
            df += sa;
            di = calc_wound(df);
            if (di < d0) {
                WOUND = df;
                d0 = di;
            } else
                break;
            if (WOUND < 0 || WOUND > 48)
                break;
        }
    }
    public double calc_wound(int ww) {
        Inha = makeCosh(ww, A49I, GRADE);

        return diff_h();
    }
    //
    public double[] makeCosh(int ww, double a49, double gr) {
        double[] tmp = new double[KB];

	double df = a49/get_cosh(48, ww, gr);
        for (int i = 0; i < KB; i++)
            tmp[i] = get_cosh(i, ww, gr)*df;

        return tmp;
    }
    public double get_cosh(int key, int ww, double grd) {
        return cosh((key-ww+1)*grd);
    }
    public double cosh(double x) {
        double me = Math.pow(Math.E, x);
        return (me+1/me)/2;
    }

    //--- sinh
    public void tuneSinh() {
	double dd = -0.005;   // Ѳ
        double min = 0.00001; // Ǿ
        double dif = 0;       // Ȥκ

	NN = 0;
	Cent = new double[KB];
        double[] ce = makeSinh();
        double me = get_sum_s(ce, NN);
        double ss, d = dd;
	while (me > 0) {
	    ss = me;
            NN -= Math.abs(d);
            me = get_sum_s(ce, NN);
            dif = ss-me;
	    if (Math.abs(dif) < min)
		break;
	    d = feedback_s(me, dif, d);
	}
	while (me < 0) {
            ss = me;
            NN += Math.abs(d);
            me = get_sum_s(ce, NN);
            dif = ss-me;
            if (Math.abs(dif) < min)
                break;
            d = feedback_s(me, dif, d);
        }
    }
    public double feedback_s(double mm, double df, double dd) {
        return 2*dd/df*mm/3;
    }
    public double get_sum_s(double[] ce, double n) {
        for (int i = 0; i < KB; i++)
            Cent[i] = ce[i]*n;

        return get_diff(get_beat());
    }
    public double get_diff(double[] be) {
        double sum = 0;
        int len = be.length;

        for (int i = 1; i < len; i++)
            sum += be[i]-be[i-1];

        return sum/len;
    }
    public double[] get_beat() {
	int ki = KB-ITV;
        double[] be = new double[ki];
        Inter k = new Inter();

        for (int i = 0; i < ki; i++) {
            k = new Inter(i, i+ITV, ITN);
            be[i] = getBeats(k);
        }

        return be;
    }
    public double getBeats(Inter n) {
        return get_ifreq(n.getHkey(), n.getLint())-
            get_ifreq(n.getLkey(), n.getHint());
    }
    public double get_ifreq(int key, int pr) {
        return ctof(getFreq(key)*pr, get_icent(key, pr));
    }
    public double get_icent(int key, int pr) {
        return Inha[key]*pr*pr+Cent[key];
    }
    public double ctof(double f, double c) {
	return f*Math.pow(2, c/CENT);
    }
    public double getFreq(int key) {
        return PITCH*Math.pow(2,(key-48)*RevO);
    }
    //
    public double[] makeSinh() {
        double[] tmp = new double[KB];

        for (int i = 0; i < KB; i++)
            tmp[i] = get_sinh(i);

        return tmp;
    }
    public double[] makeSinh(double n) {
        double[] tmp = new double[KB];

        for (int i = 0; i < KB; i++)
            tmp[i] = get_sinh(i)*n;

        return tmp;
    }
    public double get_sinh(int key) {
        return sinh((key-KC)*GRADE);
    }
    public double sinh(double x) {
        double me = Math.pow(Math.E, x);
        return (me-1/me)/2;
    }

    //--- e
    public void tuneE() {
  	AA = 0;
        BB = -GRADE*0.6;
	Ee = new double[KB];
	double min = 0.001;
        double dc = 0.005;  // 
        double d = dc;      // Ѳ
        double[] ce = makeSinh(NN);
	double a49 = ce[48];
	double ss, df = 0;
        double s = get_sum_e(a49, ce);
        while(s < BEAT) {
            AA -= d;
            ss = s;
            s = get_sum_e(a49, ce);
            df = ss-s;
            if (Math.abs(df) < min)
                break;
            d = -feedback_e(s, df, d);
	}
	while(s > BEAT) {
            AA += d;
            ss = s;
            s = get_sum_e(a49, ce);
            df = ss-s;
            if (Math.abs(df) < min)
                break;
            d = -feedback_e(s, df, d);
	}
    }
    public double feedback_e(double s, double df, double dd) {
        return dd/df*(BEAT-s)/2;
    }
    public double get_sum_e(double a49, double[] ce) {
        double[] e0 = makeE(a49);

        for (int i = 0; i < KB; i++) {
            Cent[i] = ce[i]+e0[i];
	}
        return get_mean(get_beat());
    }
    public double get_mean(double[] be) {
        double sum = 0;
        int len = be.length;

        for (int i = 0; i < len; i++)
            sum += be[i];

        return sum/len;
    }

    public double[] makeE(double a49) {
	double a0 = calcE(48)+a49;

        for (int i = 0; i < KB; i++) {
	    Ee[i] = calcE(i)-a0;
	}
        return Ee;
    }
    public double calcE(int key) {
	return AA*Math.pow(Math.E, BB*(key-KC));
    }

    //---
    public String toString() {
        StringBuffer sb = new StringBuffer();
	sb.append("GRADE:"+dform(GRADE));
        sb.append(" A49I:"+dform(A49I));
        sb.append(" WOUND:"+WOUND);
	/*
	sb.append(" NN:"+dform(nn));
	sb.append(" AA:"+dform(aa));
	sb.append(" BB:"+dform(bb));
	*/
        return sb.toString();
    }
    public String dform(double n) {
        return df1.format((float)n);
    }
}