/* $Id: TuningKeyboard.java,v 1.11 2017/03/15 08:26:31 mit3 Exp $ */
/* TuningKeyboard.java v0.4 (design) */

public class TuningKeyboard implements Runnable
{
    private boolean Runsw;
    private int KB = 88;

    private int[] OCT;
    private int[] P5TH;
    private int[] P4TH;

    private Wire[] G;
    private Thread thr;
    private Interval I = new Interval();

    private final int fromKey = 32-1;
    private final int toKey = 44-1;
    private final int OCTAVE = 12;
    private final int OCT2 = OCTAVE*2;
    private final double REVOCT = 1.0/OCTAVE;
    private final double E = 2.15*100000; // [N/mm^2]

    public TuningKeyboard() {}
    public TuningKeyboard(int kb) {
	KB = kb;
        G = new Wire[KB];

        for (int i = 0; i < KB; i++)
            G[i] = new Wire(i);

	OCT = I.getIdata(0);   // 2,1,12
        P5TH = I.getIdata(17); // 3,2,7
        P4TH = I.getIdata(13); // 4,3,5
    }
    //
    public void startRun(double be) {
        thr = new Thread(this);
        thr.start();

        Runsw = true;
    }
    public boolean isRun() {
        return Runsw;
    }
    public void stop() {
        if (thr != null) {
            thr = null;
            Runsw = false;
        }
    }
    public void run() {
        int stime = 10;              // ꡼׻

	try {
	    tuning();
	
	    Thread.sleep(stime);
	}
        catch (InterruptedException e) {
            System.out.println("Run error."+e);
        }

	Runsw = false;
    }
    public void tuning() {
	tuneLaying();

	tuneBass();
	tuneTreble();
    }
    public void tuneLaying() {
        double sa = 1.9;
        double df = sa / OCTAVE;
        double sf = 5 * df;
        double cnt = 0;
        double val;

        for (int i = fromKey; i < toKey; i++) {
            val = cnt * df - sf;
            setCent(i, val);
            cnt += 1;
        }
    }
    public void tuneBass() {
        for (int i = fromKey-1; i >= 0; i--) {
            tune_bass(i);
        }
    }
    public void tune_bass(int key) {
        int keyh0 = key + OCT[2];
        int keyh5 = key + P5TH[2];
        int keyh4 = key + P4TH[2];
        double dc = 0.1;
        double minc = 0;
        double minb = 10000;
        double dif;
        double minx = (getInha(key) > 3.0) ? minb : 1.1; // ϰϰʾ
        double maxc = (getInha(key) > 3.0) ? 100 : 60; // "

        while (true) {
            dif = getTriadsBass(key, keyh0, keyh5, keyh4);
           
            if (dif < minx) {
                minb = dif;
                minc = getCent(key);
            }
            if (getCent(key) > maxc)
                break;

            addCent(key, dc);
        }

	setCent(key, minc);
        while (true) {
            dif = getTriadsBass(key, keyh0, keyh5, keyh4);

            if (dif < minx) {
                minb = dif;
                minc = getCent(key);
            }
            if (getCent(key) < -maxc)
                break;

            addCent(key, -dc);
        }

        setCent(key, minc);
    }
    public double getTriadsBass(int key, int ko, int k5, int k4) {
        return Math.abs( get_ibeats(ko, key, OCT)+
                         get_ibeats(k5, key, P5TH)+
                         get_ibeats(k4, key, P4TH) );
	    
    }
    public double get_ibeats(int keyh, int keyl, int[] ratio) {
        double fh = get_ifreq(keyh, ratio[1]);
        double fl = get_ifreq(keyl, ratio[0]);

        return (fh - fl);
    }
    public double get_ifreq(int key, int par) {
        return c2f(getFreq(key)*par, get_icent(key, par));
    }
    public double c2f(double f, double c) {
        return f * Math.pow(2, c / 1200);
    }
    public double getFreq(int key) { // 0~87
        return 440.0 * Math.pow(2, (key - 48)*REVOCT);
	}
    public double get_icent(int key, int par) {
        return getCent(key) + calcInha(key, par);
    }
    public double calcInha(int key, int par) {
        return getInha(key) * par * par;
    }
    public void tuneTreble() {
        for (int i = toKey; i < KB; i++) {
            tune_treble(i);
        }
    }
    public void tune_treble(int key) {
        int keyl0 = key - OCT[2];
        int keyl5 = key - P5TH[2];
        int keyl4 = key - P4TH[2];
        int max = 60;
        double dc = 0.1;
        double limit = 1.1;
        double minc = 0;
        double minb = 10000;
        double dif;
	int k = 60;

        while (true) {
            dif = getTriadsTreble(key, keyl0, keyl5, keyl4);

            if (dif < limit) {
                minb = dif;
                minc = getCent(key);
            }
            if (getCent(key) < -max)
                break;
            addCent(key, -dc);
        }
	setCent(key, minc);

        while (true) {
            dif = getTriadsTreble(key, keyl0, keyl5, keyl4);

            if (dif < limit) {
                minb = dif;
                minc = getCent(key);
            }
            if (getCent(key) > max)
                break;
            addCent(key, dc);
        }

        setCent(key, minc);
    }
    public double getTriadsTreble(int key, int ko, int k5, int k4) {
	double p5 = get_ibeats(key, k5, P5TH);
	double p4 = get_ibeats(key, k4, P4TH);
	double oc = get_ibeats(key, ko, OCT);
	double ab = Math.abs(p5+p4+oc);

	return ab;
    }
    //
    public void addCent(int kn, double ce) {
	G[kn].addCent(ce);
    }
    public void setCent(int kn, double ce) {
	G[kn].setCent(ce);
    }
    public double getCent(int kn) {
        return G[kn].getCent();
    }
    public double getIcent(int kn, int pr) {
	return getInha(kn, pr)+getCent(kn);
    }
    public double getIfreq(int kn, int pr) {
	return ctof(getFreq(kn)*pr, getInha(kn, pr));
    }
    public double getInha(int kn, int pr) {
        return G[kn].getInhar()*pr*pr;
    }
    public double getInha(int kn) {
	return G[kn].getInhar();
    }
    public double ctof(double f, double ce) {
	return f*Math.pow(2, ce/1200);
    }
    //
    public void setLength(int k, double le) {
        G[k].setLength(le);
    }
    public void setGauge(int k, double ga) {
        G[k].setGauge(ga);
    }
    public void setCopper(int k, double co) {
        G[k].setCopper(co);
    }
    public void setWire(int k, double le, double ga, double co) {
        G[k].setWire(le, ga, co);
    }
    public double getLength(int k) {
        return G[k].getLength();
    }
    public double getGauge(int k) {
        return G[k].getGauge();
    }
    public double getCopper(int k) {
        return G[k].getCopper();
    }
    public double getDensity(int k) {
	return G[k].getDensity();
    }
    public double getGTension(int k) {
        return G[k].getGTension();
    }
    public double getTension(int k) {
        return G[k].getTension();
    }
    public double getDia(int k) {
        return G[k].getDia();
    }
    public double getDiameter(int k) {
        return G[k].getDiameter();
    }
    public double getAlld(int k) {
        return G[k].getAlld();
    }
    public double getalld(int k) {
        return G[k].getalld();
    }
    public double getLoad(int k) {
        return G[k].getLoad();
    }
    public double getElongation(int k) {
        return G[k].getElongation(E);
    }
    public double getStress(int k) {
        return G[k].getStress();
    }
    public String toString(int k) {
        return G[k].toString();
    }
    public int getNum(int k) {
        return G[k].getKeynum();
    }
    public boolean isCopper(int k) {
        return G[k].isCopper();
    }
    public Wire[] getG() {
	return G;
    }
    public double getBanteMin() {
	return G[0].getBanteMin();
    }
    public double getBanteMax() {
	return G[0].getBanteMax();
    }
}
